import argparse
import os
import string
import random
import glob
import subprocess

def create_parser():
    parser = argparse.ArgumentParser(
        prog="qlprint",
        description="Printing tools for Brother QL-570 label printer",
        epilog="Have fun printing!")
    parser.add_argument("image", type=str, help="path to image (with extension, i.e. .png) or name of temporary folder to continue printing part way through a failed print job (this would be a random string of 10 letters or numbers, e.g. oet845cgx9)")
    parser.add_argument("-c", "--copies", type=int, default=1, help="number of copies of the image/images to be printed")
    parser.add_argument("-t", "--test", type=bool, default=False, help="run program without printing")
    return parser

def qlprint():
    parser = create_parser()
    args = parser.parse_args()
    if "." in args.image:
        name = "".join(random.choices(string.ascii_lowercase + string.digits, k=10))
        path = f"/tmp/{name}"
        print(f"Creating temporary directory at {path}. To recover failed prints please pass the recovery key {name}")
        os.mkdir(path)
        filename = f"{path}/{name}.png"
        print("Converting images")
        task = subprocess.Popen(["magick", "-density", "300", args.image, "-trim", "-resize", "696x", filename])
        task.wait()
        do_print = True                                                                                            
    else:
        path = f"/tmp/{args.image}"
        name = args.image
        if not os.path.isdir(path):
            print(f"{path} does not exist. If you are trying to continue printing can you check that the recovery key is correct. If you are trying to print a new image perhaps the extension is missing from filename (.png, .pdf, etc)")
            do_print = False
        elif len(os.listdir(path)) == 0:
            print(f"{path} exists but is empty - there is a chance that everything has already been printed. If not and if you are trying to continue printing can you check that the recovery key is correct. If you are trying to print a new image perhaps the extension is missing from filename (.png, .pdf, etc)")
            do_print = False
        else:
            print(f"Continuing print from {path}")
            do_print = True
    if do_print:
        if len(os.listdir(path)) > 1:
            files = sorted(glob.glob(f"{path}/{name}*.png"), key=lambda arg: int(os.path.splitext(os.path.basename(arg))[0].split("-")[1]))
        else:
            files = [f"{path}/{name}.png"]
        for file in files:
            for i in range(args.copies):
                print(f"Printing copy {i + 1} of {os.path.basename(file)}")
                if not args.test:
                    task = subprocess.Popen([f"brother_ql -b pyusb -m QL-570 -p usb://0x04f9:0x2028/J5Z416681 print -l 62 {file}"], stderr=subprocess.PIPE, shell=True)
                    task.wait()
                    stderr = task.stderr.read().decode().split("\n")[-2]
                    if stderr != "INFO:brother_ql.backends.helpers:Printing was successful. Waiting for the next job.":
                        print("Print failed... continuing")
                    if i + 1 == args.copies:
                        if stderr == "INFO:brother_ql.backends.helpers:Printing was successful. Waiting for the next job.":
                            os.remove(file)
        if len(os.listdir(path)) == 0:
            print("Removing temporary directory")
            os.rmdir(path)

def main():
    qlprint()
    
if __name__ == "__main__":
    main()
        
            
        
            
            
        
                    
        
